/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DiscRackGateway.java,v 1.5 2004/06/04 19:12:14 slobodan Exp $
 */
package org.enhydra.barracuda.discRack;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import com.lutris.appserver.server.Application;
import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.StandardApplication;
import com.lutris.logging.StandardLogger;
import com.lutris.util.Config;
import com.lutris.util.ConfigFile;

import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.apache.log4j.*;

import org.enhydra.barracuda.discRack.pres.screens.*;
//import org.enhydra.barracuda.examples.ex2.ConfigScreen;

//import org.enhydra.instantdb.jdbc.idbDriver;

/**
 * <p>Application Gateway for the Barracuda DiscRack example. Couple of
 * special notes regarding deployment.
 *
 * <p>1. This example was built using Enhydra 3.1 and XMLC 1.2. For
 *	  best results, use these versions or higher.
 *
 * <p>2. This example uses Instant DB and the standard DODS generated
 *    data classes from previous DiscRack examples. To make it work,
 *	  you will need to add 3 jar files to the enhydra classpath:
 *		<ul>
 *		 <li>idb.jar (v3.21)</li>
 *		 <li>jdbc2_0-stdext.jar</li>
 *		 <li>jta-spec1_0_1.jar</li>
 *		</ul>
 *
 * <p>We can't just put these files in /Web-inf/lib because DODs is
 *	  dependant on the enhydra Application interface, and the
 *	  StandardApplication implementation requires these jar files
 * 	  be part of the enhydra classpath
 *
 * <p>3. You must manually build/populate the IDB databases in /Web-inf/db.
 * 	  To do this use the idb ScriptTool on sql1.txt (in the db directory)
 *
 * <p>To understand this example, you should really look at the class diagram
 * which illustrates how everythign fits together.
 * <a href="sm_barracudaDiscRack.gif">You can view it here...</a>
 *
 */
public class DiscRackGateway extends ApplicationGateway {

	//public constants
	public static final String ENHYDRA_APPLICATION = DiscRackGateway.class.getName()+"EnhydraApplication";
	protected static Logger logger = Logger.getLogger(DiscRackGateway.class.getName());

	//private vars
	private static final String CONFIG_FILE = "ConfFile";
//	private ServletConfig config = null;
    private LoginScreen lnkLoginScreen;						//added by TJ to show association
    private RegistrationScreen lnkRegistrationScreen;		//added by TJ to show association
    private DiscScreen lnkDiscScreen;						//added by TJ to show association



	//-------------------- ApplicationGateway --------------------
	/**
	 * Perform any local initialization (this is where you should
	 * specify any known EventGateways)
	 */
	public void initializeLocal() {
		//set up the enhydra app for DODs stuff
//		this.config = config;
		setUpEnhydraApplication();

		//specify the event gateways
		this.specifyEventGateways(new LoginScreen());
		this.specifyEventGateways(new RegistrationScreen());
		this.specifyEventGateways(new DiscScreen());
	}


	/**
	 * Set up an underlying Enhydra application for database
	 * related stuff (DODS needs this to function properly).
	 * We'll store a reference to the Application in the
	 * ServletContext, so if anyone needs access to it, they
	 * can get it.
	 */
//	private void setUpEnhydraApplication(ServletConfig config) {
	private void setUpEnhydraApplication() {
		//get the context and see if the app is already set
		ServletContext context =  this.getServletConfig().getServletContext();
		Application app = (Application) context.getAttribute(ENHYDRA_APPLICATION);

		//if not, create it...
		if (app==null) {
			try {
				//create the underlying enhydra Application
				if (logger.isDebugEnabled()) logger.debug("Creating underlying Enhydra Application");
				app = new DiscRackApplication();

				//set the log channel
				app.setLogChannel(new StandardLogger(true).getChannel(""));

				//get the config file
				String configFile = this.getServletConfig().getInitParameter(CONFIG_FILE);

				if (logger.isDebugEnabled()) logger.debug("Config:"+configFile);

				//InputStream is = this.getClass().getResourceAsStream(configFile);
        //InputStream is = context.getResourceAsStream(configFile);
        //if (is==null) is = this.getClass().getResourceAsStream(configFile);
        
        String is = context.getRealPath(configFile);
				File isf = new File (is);

				Config conf = new ConfigFile(isf).getConfig();
				if (logger.isDebugEnabled()) logger.debug("Got Config file:"+conf);

				//startup the app
				app.startup(conf);
				if (logger.isDebugEnabled()) logger.debug("Started app:"+app);

				//save it in the context
				context.setAttribute(ENHYDRA_APPLICATION, app);
			} catch(Exception e) {
				System.out.println("Error setting up Enhydra Application:"+e);
				e.printStackTrace();
			}
		}
	}

	/**
	 * Handle all HTTP requests
	 */
	public void handleDefault (HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		ServletContext context =  this.getServletConfig().getServletContext();
		Application app = (Application) context.getAttribute(ENHYDRA_APPLICATION);
		try {
			if (logger.isDebugEnabled()) logger.debug("\n\n"+req.getRequestURI()+" @"+Integer.toHexString(req.hashCode()));
			Enhydra.register(app);
			super.handleDefault(req, resp);
		} finally {
			Enhydra.unRegister();
		}
	}

	class DiscRackApplication extends StandardApplication {

	}

}
